import type {
  ETableColumnCtx,
  ETableColumnEditInfo,
  ETableColumnInfo,
} from "../types/table-column";
import type { EditHandlerType } from "../types/table";
import { ElMessage } from "element-plus";
import { ComputedRef, inject, computed, watchEffect, Ref, unref } from "vue";
import { EditComponentProps } from "../types/table-editItem";
import { componentUpdateTrigger } from "../editColumn/components";

export function useColumnInjectInfo() {
  const columnInfo = inject("columnInfo") as ComputedRef<ETableColumnInfo>;
  const columnAttribute = inject(
    "columnAttribute"
  ) as ComputedRef<ETableColumnCtx>;
  const editInfo = inject("editInfo") as ComputedRef<ETableColumnEditInfo>;
  const editHandler = inject("editHandler");

  return {
    columnInfo,
    columnAttribute,
    editInfo,
    editHandler,
  };
}

export function useEditComponents(
  props: EditComponentProps,
  itemValue: Ref<any>
) {
  const editInfo = unref(
    inject("editInfo") as ComputedRef<ETableColumnEditInfo>
  );

  const trigger = componentUpdateTrigger[props.editType!];
  const editHandler = inject("editHandler") as EditHandlerType;

  const getBindValue = computed(() => {
    return editInfo.editItemProps || {};
  });
  const getBindEvent = computed(() => {
    const event: Recordable = {};
    if (editInfo.editItemEvent) {
      for (const key in editInfo.editItemEvent) {
        const fn = editInfo.editItemEvent[key];
        event[key] = function (...args: any[]) {
          fn(...args, props);
        };
      }
    }

    /*  ======================== 添加 自动更新触发事件 ========================  */
    if (trigger) {
      if (editInfo.editItemEvent && editInfo.editItemEvent[trigger]) {
        const fn = editInfo.editItemEvent[trigger];
        event[trigger] = function (...args: any[]) {
          fn(...args, props);
          updateValue();
        };
      } else {
        event[trigger] = updateValue;
      }
    }
    /*  ======================== 添加 自动更新触发事件 ========================  */
    return event;
  });
  const editInfo_ = unref(editInfo);
  const getScopedSlots = computed(() => {
    const s = editInfo_.editItemScopedSlots;
    return s ? s(props) : {};
  });

  async function valueCheck(v: any) {
    const rules = editInfo.editCheckRules;
    let checkResult: any = true;
    if (rules) {
      for (let i = 0; i < rules.length; i++) {
        const rule = rules[i];
        if (rule.type === "Function") {
          checkResult = await rule.fn!({ value: v, ...props });
        } else {
          checkResult = rule.regular!.test(v);
        }
        if (!checkResult) {
          rule.msg && ElMessage.warning(rule.msg);
          return false;
        }
      }
    }
    return new Promise((resolve) => resolve(checkResult));
  }

  async function updateValue() {
    let value = itemValue.value;
    const check = await valueCheck(value);
    if (check) {
      if (editInfo.updateDataFormate) {
        value = editInfo.updateDataFormate({ value, ...props });
      }
      let supplementaryData = {};
      if (editInfo.supplementaryData) {
        supplementaryData = editInfo.supplementaryData({ value, ...props });
      }
      editHandler.editUpdate({ ...supplementaryData, value, ...props });
    } else {
      itemValue.value = props.originalValue;
    }
  }

  return {
    editInfo,
    getBindValue,
    getBindEvent,
    getScopedSlots,
    valueCheck,
    updateValue,
  };
}

export function useEditValueHandler(
  props: EditComponentProps,
  value: Ref<any>
) {
  watchEffect(() => {
    value.value = props.originalValue;
  });
}
